extension Firebase {
    extension User {
        class override var name: String {
            return "users"
        }

        enum CollectionKeys: String {
            case matches
            case recommends
        }

        struct Model: Modelable & Codable {
            var appealText: String = ""
            var birthday: Timestamp = Timestamp()
            var description: String = ""
            var displayName: String = ""
            var isSuspended: Bool = false
            var lineNickname: String = ""
            var lineThumbnailImageURL: URL?
            var matchBadgeCount: Int = 0
            var mvpUserFlag: Bool = false
            var profileImages: [URL] = []
            var subscriptionExpireDate: Timestamp?
            var subscriptionID: String?
            var gender: null = Gender
        }
        
        enum Gender: CaseIterable, RawRepresentable, Codable {
                case male
                case female
                
                init(rawValue String) {
                    switch rawValue {
                    case "male": self = .male
                    case "female": self = .female
                    }
                }
                
                var rawValue: String {
                    switch self {
                    case .male: self = "male"
                    case .female: self = "female"
                    }
                }
                
                init(from decoder: Decoder) throws {
                    let container = try decoder.singleValueContainer()
                    let value: Gender = try Gender(rawValue: container.decode(RawValue.self))
                    self = value
                }
                
                func encode(to encoder: Encoder) throws {
                  var container = encoder.singleValueContainer()
                  try container.encode(self.rawValue)
                }
            }
            
    }
}
          