import 'package:cloud_firestore/cloud_firestore.dart' as _i1;
import 'package:firebase_storage/firebase_storage.dart' as _i2;
import 'package:mime/mime.dart' as _i3;
import 'dart:io' as _i4;
import 'package:meta/meta.dart' as _i5;

_i1.Firestore _firestore = _i1.Firestore.instance;
void setFirestoreInstance(_i1.Firestore instance) {
  _firestore = instance;
}

_i2.FirebaseStorage _storage = _i2.FirebaseStorage.instance;
void setFirebaseStorageInstance(_i2.FirebaseStorage instance) {
  _storage = instance;
}

typedef DartTypeConverter<T, U> = T Function(U);
T _idConverter<T>(T v) {
  return v;
}

DateTime _timestampToDateTimeConverter(dynamic v) {
  return v?.toDate();
}

Uri _stringUrlToUriConverter(dynamic v) {
  return (v != null) ? Uri.parse(v) : null;
}

int _numToIntConverter(dynamic v) {
  return v?.toInt();
}

FileReference _fileMapToFileReferenceConverter(dynamic v) {
  return (v != null) ? _RemoteFile._(v) : null;
}

Map<Type, DartTypeConverter> _dartTypeConverterMap = {
  DateTime: _timestampToDateTimeConverter,
  Uri: _stringUrlToUriConverter,
  int: _numToIntConverter,
  FileReference: _fileMapToFileReferenceConverter,
  Gender: _stringToEnumGenderConverter
  // TODO: Converter for Geopoint
};
T _convertDartType<T>(dynamic v) {
  return (_dartTypeConverterMap[T] ?? _idConverter).call(v);
}

typedef FirestoreStructureConverter<T, U> = Future<T> Function(U, String);
Future<T> _delayedIdConverter<T>(T v, String documentPath) async {
  return v;
}

Future<Map<String, dynamic>> _fileReferenceToFileMapConverter(
    dynamic v, String documentPath) async {
  if (v is _LocalFile) {
    final magic = await v._file.openRead(0, 1).first;
    final mimeType = _i3.lookupMimeType(v._file.path, headerBytes: magic);
    final ref = _storage.path(documentPath);
    await ref
        .putFile(v._file, _i2.StorageMetadata(contentType: mimeType))
        .onComplete;
    return {
      "additionlData": <String, dynamic>{},
      "mimeType": mimeType,
      "path": documentPath,
      "url": await ref.getDownloadURL(),
    };
  } else {
    return (v as _RemoteFile)._fileStructure;
  }
}

Map<Type, FirestoreStructureConverter> _firestoreStructureConverterMap = {
  FileReference: _fileReferenceToFileMapConverter,
  Gender: _enumGenderToStringConverter
};
Future<dynamic> _convertFirestoreStructure<T>(T v, String documentPath) {
  return (_firestoreStructureConverterMap[T] ?? _delayedIdConverter)
      .call(v, documentPath);
}

abstract class FileReference {
  factory FileReference.local(_i4.File file) {
    return _LocalFile._(file);
  }

  Uri get uri;
}

class _LocalFile implements FileReference {
  _LocalFile._(this._file) : assert(_file != null);

  final _i4.File _file;

  @override
  Uri get uri {
    return _file.uri;
  }
}

class _RemoteFile implements FileReference {
  _RemoteFile._(this._fileStructure) : assert(_fileStructure != null);

  final Map<String, dynamic> _fileStructure;

  @override
  Uri get uri {
    return Uri.parse(_fileStructure["url"]);
  }
}

class versionCollection {
  versionCollection(this.reference) : assert(reference != null);

  factory versionCollection.fromPath(String path) {
    return versionCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  version__nonamedocument__Document documentById(String id) {
    return version__nonamedocument__Document(reference.document(id));
  }
}

class version__nonamedocument__Document {
  version__nonamedocument__Document(this.reference) : assert(reference != null);

  factory version__nonamedocument__Document.fromPath(String path) {
    return version__nonamedocument__Document(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  versionCollection get parent => versionCollection(reference.parent());
  Future<version__nonamedocument__Snapshot> getSnapshot() {
    return reference
        .get()
        .then((s) => version__nonamedocument__Snapshot.fromSnapshot(s));
  }

  Stream<version__nonamedocument__Snapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => version__nonamedocument__Snapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<version__nonamedocument__Snapshot> create() {
    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{};
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => version__nonamedocument__Snapshot());
  }

  Future<version__nonamedocument__Snapshot> update() {
    return reference.updateData({}).then((_) => getSnapshot());
  }

  usersCollection get users {
    return usersCollection(reference.collection("users"));
  }

  secureusersCollection get secureusers {
    return secureusersCollection(reference.collection("secureusers"));
  }

  roomgroupCollection get roomgroup {
    return roomgroupCollection(reference.collection("roomgroup"));
  }
}

class version__nonamedocument__Snapshot {
  const version__nonamedocument__Snapshot();

  factory version__nonamedocument__Snapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return version__nonamedocument__Snapshot();
    } else {
      return null;
    }
  }
}

class usersCollection {
  usersCollection(this.reference) : assert(reference != null);

  factory usersCollection.fromPath(String path) {
    return usersCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  UserDocument documentById(String id) {
    return UserDocument(reference.document(id));
  }
}

class UserDocument {
  UserDocument(this.reference) : assert(reference != null);

  factory UserDocument.fromPath(String path) {
    return UserDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  usersCollection get parent => usersCollection(reference.parent());
  Future<UserSnapshot> getSnapshot() {
    return reference.get().then((s) => UserSnapshot.fromSnapshot(s));
  }

  Stream<UserSnapshot> snapshotUpdates({bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => UserSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<UserSnapshot> create(
      {@_i5.required String appealText,
      @_i5.required DateTime birthday,
      @_i5.required String description,
      @_i5.required String displayName,
      @_i5.required bool isSuspended,
      @_i5.required String lineNickname,
      Uri lineThumbnailImageURL,
      @_i5.required int matchBadgeCount,
      @_i5.required bool mvpUserFlag,
      @_i5.required List<Uri> profileImages,
      DateTime subscriptionExpireDate,
      String subscriptionID,
      @_i5.required Gender gender}) {
    assert(appealText != null);
    assert(birthday != null);
    assert(description != null);
    assert(displayName != null);
    assert(isSuspended != null);
    assert(lineNickname != null);
    assert(matchBadgeCount != null);
    assert(mvpUserFlag != null);
    assert(profileImages != null);
    assert(gender != null);

    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "appealText": _convertFirestoreStructure(appealText, reference.path),
      "birthday": _convertFirestoreStructure(birthday, reference.path),
      "description": _convertFirestoreStructure(description, reference.path),
      "displayName": _convertFirestoreStructure(displayName, reference.path),
      "isSuspended": _convertFirestoreStructure(isSuspended, reference.path),
      "lineNickname": _convertFirestoreStructure(lineNickname, reference.path),
      "lineThumbnailImageURL":
          _convertFirestoreStructure(lineThumbnailImageURL, reference.path),
      "matchBadgeCount":
          _convertFirestoreStructure(matchBadgeCount, reference.path),
      "mvpUserFlag": _convertFirestoreStructure(mvpUserFlag, reference.path),
      "profileImages":
          _convertFirestoreStructure(profileImages, reference.path),
      "subscriptionExpireDate":
          _convertFirestoreStructure(subscriptionExpireDate, reference.path),
      "subscriptionID":
          _convertFirestoreStructure(subscriptionID, reference.path),
      "gender": _convertFirestoreStructure(gender, reference.path),
      "createdAt": _convertFirestoreStructure(createdAt, reference.path),
      "modifiedAt": _convertFirestoreStructure(modifiedAt, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => UserSnapshot(
            appealText: appealText,
            birthday: birthday,
            description: description,
            displayName: displayName,
            isSuspended: isSuspended,
            lineNickname: lineNickname,
            lineThumbnailImageURL: lineThumbnailImageURL,
            matchBadgeCount: matchBadgeCount,
            mvpUserFlag: mvpUserFlag,
            profileImages: profileImages,
            subscriptionExpireDate: subscriptionExpireDate,
            subscriptionID: subscriptionID,
            gender: gender,
            createdAt: createdAt,
            modifiedAt: modifiedAt));
  }

  Future<UserSnapshot> update(
      {String appealText,
      DateTime birthday,
      String description,
      String displayName,
      bool isSuspended,
      String lineNickname,
      Uri lineThumbnailImageURL,
      int matchBadgeCount,
      bool mvpUserFlag,
      List<Uri> profileImages,
      DateTime subscriptionExpireDate,
      String subscriptionID,
      Gender gender}) {
    return reference.updateData({
      "appealText": appealText,
      "birthday": birthday,
      "description": description,
      "displayName": displayName,
      "isSuspended": isSuspended,
      "lineNickname": lineNickname,
      "lineThumbnailImageURL": lineThumbnailImageURL,
      "matchBadgeCount": matchBadgeCount,
      "mvpUserFlag": mvpUserFlag,
      "profileImages": profileImages,
      "subscriptionExpireDate": subscriptionExpireDate,
      "subscriptionID": subscriptionID,
      "gender": gender,
      "modifiedAt": DateTime.now()
    }).then((_) => getSnapshot());
  }

  matchesCollection get matches {
    return matchesCollection(reference.collection("matches"));
  }

  recommendsCollection get recommends {
    return recommendsCollection(reference.collection("recommends"));
  }
}

class UserSnapshot {
  const UserSnapshot(
      {@_i5.required this.appealText,
      @_i5.required this.birthday,
      @_i5.required this.description,
      @_i5.required this.displayName,
      @_i5.required this.isSuspended,
      @_i5.required this.lineNickname,
      this.lineThumbnailImageURL,
      @_i5.required this.matchBadgeCount,
      @_i5.required this.mvpUserFlag,
      @_i5.required this.profileImages,
      this.subscriptionExpireDate,
      this.subscriptionID,
      @_i5.required this.gender,
      @_i5.required this.createdAt,
      @_i5.required this.modifiedAt})
      : assert(appealText != null),
        assert(birthday != null),
        assert(description != null),
        assert(displayName != null),
        assert(isSuspended != null),
        assert(lineNickname != null),
        assert(matchBadgeCount != null),
        assert(mvpUserFlag != null),
        assert(profileImages != null),
        assert(gender != null),
        assert(createdAt != null),
        assert(modifiedAt != null);

  factory UserSnapshot.fromSnapshot(_i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return UserSnapshot(
          appealText: _convertDartType(documentSnapshot["appealText"]),
          birthday: _convertDartType(documentSnapshot["birthday"]),
          description: _convertDartType(documentSnapshot["description"]),
          displayName: _convertDartType(documentSnapshot["displayName"]),
          isSuspended: _convertDartType(documentSnapshot["isSuspended"]),
          lineNickname: _convertDartType(documentSnapshot["lineNickname"]),
          lineThumbnailImageURL:
              _convertDartType(documentSnapshot["lineThumbnailImageURL"]),
          matchBadgeCount:
              _convertDartType(documentSnapshot["matchBadgeCount"]),
          mvpUserFlag: _convertDartType(documentSnapshot["mvpUserFlag"]),
          profileImages: _convertDartType(documentSnapshot["profileImages"]),
          subscriptionExpireDate:
              _convertDartType(documentSnapshot["subscriptionExpireDate"]),
          subscriptionID: _convertDartType(documentSnapshot["subscriptionID"]),
          gender: _convertDartType(documentSnapshot["gender"]),
          createdAt: _convertDartType(documentSnapshot["createdAt"]),
          modifiedAt: _convertDartType(documentSnapshot["modifiedAt"]));
    } else {
      return null;
    }
  }

  final String appealText;

  final DateTime birthday;

  final String description;

  final String displayName;

  final bool isSuspended;

  final String lineNickname;

  final Uri lineThumbnailImageURL;

  final int matchBadgeCount;

  final bool mvpUserFlag;

  final List<Uri> profileImages;

  final DateTime subscriptionExpireDate;

  final String subscriptionID;

  final Gender gender;

  final DateTime createdAt;

  final DateTime modifiedAt;
}

class Gender {
  const Gender._(this.index, this.value);

  factory Gender.fromValue(String value) =>
      values.where((v) => v.value == value).first;

  final String value;

  final int index;

  static const male = Gender._(0, "male");

  static const female = Gender._(1, "female");

  static const values = [male, female];

  @override
  String toString() => "Gender.$value";
}

Gender _stringToEnumGenderConverter(dynamic v) => Gender.fromValue(v);
Future<String> _enumGenderToStringConverter(dynamic v, String _) async {
  return v.value;
}

class matchesCollection {
  matchesCollection(this.reference) : assert(reference != null);

  factory matchesCollection.fromPath(String path) {
    return matchesCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  MatchDocument documentById(String id) {
    return MatchDocument(reference.document(id));
  }
}

class MatchDocument {
  MatchDocument(this.reference) : assert(reference != null);

  factory MatchDocument.fromPath(String path) {
    return MatchDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  matchesCollection get parent => matchesCollection(reference.parent());
  Future<MatchSnapshot> getSnapshot() {
    return reference.get().then((s) => MatchSnapshot.fromSnapshot(s));
  }

  Stream<MatchSnapshot> snapshotUpdates({bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => MatchSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<MatchSnapshot> create() {
    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "createdAt": _convertFirestoreStructure(createdAt, reference.path),
      "modifiedAt": _convertFirestoreStructure(modifiedAt, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then(
            (_) => MatchSnapshot(createdAt: createdAt, modifiedAt: modifiedAt));
  }

  Future<MatchSnapshot> update() {
    return reference
        .updateData({"modifiedAt": DateTime.now()}).then((_) => getSnapshot());
  }
}

class MatchSnapshot {
  const MatchSnapshot(
      {@_i5.required this.createdAt, @_i5.required this.modifiedAt})
      : assert(createdAt != null),
        assert(modifiedAt != null);

  factory MatchSnapshot.fromSnapshot(_i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return MatchSnapshot(
          createdAt: _convertDartType(documentSnapshot["createdAt"]),
          modifiedAt: _convertDartType(documentSnapshot["modifiedAt"]));
    } else {
      return null;
    }
  }

  final DateTime createdAt;

  final DateTime modifiedAt;
}

class recommendsCollection {
  recommendsCollection(this.reference) : assert(reference != null);

  factory recommendsCollection.fromPath(String path) {
    return recommendsCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  RecommendDocument documentById(String id) {
    return RecommendDocument(reference.document(id));
  }
}

class RecommendDocument {
  RecommendDocument(this.reference) : assert(reference != null);

  factory RecommendDocument.fromPath(String path) {
    return RecommendDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  recommendsCollection get parent => recommendsCollection(reference.parent());
  Future<RecommendSnapshot> getSnapshot() {
    return reference.get().then((s) => RecommendSnapshot.fromSnapshot(s));
  }

  Stream<RecommendSnapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => RecommendSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<RecommendSnapshot> create(
      {@_i5.required bool isAvailabled,
      @_i5.required String recommendUser,
      @_i5.required List<String> supportedUsers}) {
    assert(isAvailabled != null);
    assert(recommendUser != null);
    assert(supportedUsers != null);

    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "isAvailabled": _convertFirestoreStructure(isAvailabled, reference.path),
      "recommendUser":
          _convertFirestoreStructure(recommendUser, reference.path),
      "supportedUsers":
          _convertFirestoreStructure(supportedUsers, reference.path),
      "createdAt": _convertFirestoreStructure(createdAt, reference.path),
      "modifiedAt": _convertFirestoreStructure(modifiedAt, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => RecommendSnapshot(
            isAvailabled: isAvailabled,
            recommendUser: recommendUser,
            supportedUsers: supportedUsers,
            createdAt: createdAt,
            modifiedAt: modifiedAt));
  }

  Future<RecommendSnapshot> update(
      {bool isAvailabled, String recommendUser, List<String> supportedUsers}) {
    return reference.updateData({
      "isAvailabled": isAvailabled,
      "recommendUser": recommendUser,
      "supportedUsers": supportedUsers,
      "modifiedAt": DateTime.now()
    }).then((_) => getSnapshot());
  }
}

class RecommendSnapshot {
  const RecommendSnapshot(
      {@_i5.required this.isAvailabled,
      @_i5.required this.recommendUser,
      @_i5.required this.supportedUsers,
      @_i5.required this.createdAt,
      @_i5.required this.modifiedAt})
      : assert(isAvailabled != null),
        assert(recommendUser != null),
        assert(supportedUsers != null),
        assert(createdAt != null),
        assert(modifiedAt != null);

  factory RecommendSnapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return RecommendSnapshot(
          isAvailabled: _convertDartType(documentSnapshot["isAvailabled"]),
          recommendUser: _convertDartType(documentSnapshot["recommendUser"]),
          supportedUsers: _convertDartType(documentSnapshot["supportedUsers"]),
          createdAt: _convertDartType(documentSnapshot["createdAt"]),
          modifiedAt: _convertDartType(documentSnapshot["modifiedAt"]));
    } else {
      return null;
    }
  }

  final bool isAvailabled;

  final String recommendUser;

  final List<String> supportedUsers;

  final DateTime createdAt;

  final DateTime modifiedAt;
}

class secureusersCollection {
  secureusersCollection(this.reference) : assert(reference != null);

  factory secureusersCollection.fromPath(String path) {
    return secureusersCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  SecureUserDocument documentById(String id) {
    return SecureUserDocument(reference.document(id));
  }

  SecureUserDocument newDocument() {
    return SecureUserDocument(reference.document());
  }
}

class SecureUserDocument {
  SecureUserDocument(this.reference) : assert(reference != null);

  factory SecureUserDocument.fromPath(String path) {
    return SecureUserDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  secureusersCollection get parent => secureusersCollection(reference.parent());
  Future<SecureUserSnapshot> getSnapshot() {
    return reference.get().then((s) => SecureUserSnapshot.fromSnapshot(s));
  }

  Stream<SecureUserSnapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => SecureUserSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<SecureUserSnapshot> create(
      {@_i5.required Map<String, dynamic> fcmTokens,
      @_i5.required String lineId,
      @_i5.required String lineToken}) {
    assert(fcmTokens != null);
    assert(lineId != null);
    assert(lineToken != null);

    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "fcmTokens": _convertFirestoreStructure(fcmTokens, reference.path),
      "lineId": _convertFirestoreStructure(lineId, reference.path),
      "lineToken": _convertFirestoreStructure(lineToken, reference.path),
      "createdAt": _convertFirestoreStructure(createdAt, reference.path),
      "modifiedAt": _convertFirestoreStructure(modifiedAt, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => SecureUserSnapshot(
            fcmTokens: fcmTokens,
            lineId: lineId,
            lineToken: lineToken,
            createdAt: createdAt,
            modifiedAt: modifiedAt));
  }

  Future<SecureUserSnapshot> update(
      {Map<String, dynamic> fcmTokens, String lineId, String lineToken}) {
    return reference.updateData({
      "fcmTokens": fcmTokens,
      "lineId": lineId,
      "lineToken": lineToken,
      "modifiedAt": DateTime.now()
    }).then((_) => getSnapshot());
  }
}

class SecureUserSnapshot {
  const SecureUserSnapshot(
      {@_i5.required this.fcmTokens,
      @_i5.required this.lineId,
      @_i5.required this.lineToken,
      @_i5.required this.createdAt,
      @_i5.required this.modifiedAt})
      : assert(fcmTokens != null),
        assert(lineId != null),
        assert(lineToken != null),
        assert(createdAt != null),
        assert(modifiedAt != null);

  factory SecureUserSnapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return SecureUserSnapshot(
          fcmTokens: _convertDartType(documentSnapshot["fcmTokens"]),
          lineId: _convertDartType(documentSnapshot["lineId"]),
          lineToken: _convertDartType(documentSnapshot["lineToken"]),
          createdAt: _convertDartType(documentSnapshot["createdAt"]),
          modifiedAt: _convertDartType(documentSnapshot["modifiedAt"]));
    } else {
      return null;
    }
  }

  final Map<String, dynamic> fcmTokens;

  final String lineId;

  final String lineToken;

  final DateTime createdAt;

  final DateTime modifiedAt;
}

class roomgroupCollection {
  roomgroupCollection(this.reference) : assert(reference != null);

  factory roomgroupCollection.fromPath(String path) {
    return roomgroupCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  roomgroup__nonamedocument__Document documentById(String id) {
    return roomgroup__nonamedocument__Document(reference.document(id));
  }

  roomgroup__nonamedocument__Document newDocument() {
    return roomgroup__nonamedocument__Document(reference.document());
  }
}

class roomgroup__nonamedocument__Document {
  roomgroup__nonamedocument__Document(this.reference)
      : assert(reference != null);

  factory roomgroup__nonamedocument__Document.fromPath(String path) {
    return roomgroup__nonamedocument__Document(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  roomgroupCollection get parent => roomgroupCollection(reference.parent());
  Future<roomgroup__nonamedocument__Snapshot> getSnapshot() {
    return reference
        .get()
        .then((s) => roomgroup__nonamedocument__Snapshot.fromSnapshot(s));
  }

  Stream<roomgroup__nonamedocument__Snapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => roomgroup__nonamedocument__Snapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<roomgroup__nonamedocument__Snapshot> create() {
    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{};
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => roomgroup__nonamedocument__Snapshot());
  }

  Future<roomgroup__nonamedocument__Snapshot> update() {
    return reference.updateData({}).then((_) => getSnapshot());
  }

  shardroomsCollection get shardrooms {
    return shardroomsCollection(reference.collection("shardrooms"));
  }
}

class roomgroup__nonamedocument__Snapshot {
  const roomgroup__nonamedocument__Snapshot();

  factory roomgroup__nonamedocument__Snapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return roomgroup__nonamedocument__Snapshot();
    } else {
      return null;
    }
  }
}

class shardroomsCollection {
  shardroomsCollection(this.reference) : assert(reference != null);

  factory shardroomsCollection.fromPath(String path) {
    return shardroomsCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  ShardRoomDocument documentById(String id) {
    return ShardRoomDocument(reference.document(id));
  }
}

class ShardRoomDocument {
  ShardRoomDocument(this.reference) : assert(reference != null);

  factory ShardRoomDocument.fromPath(String path) {
    return ShardRoomDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  shardroomsCollection get parent => shardroomsCollection(reference.parent());
  Future<ShardRoomSnapshot> getSnapshot() {
    return reference.get().then((s) => ShardRoomSnapshot.fromSnapshot(s));
  }

  Stream<ShardRoomSnapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => ShardRoomSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<ShardRoomSnapshot> create(
      {@_i5.required bool isHidden,
      @_i5.required List<String> members,
      @_i5.required bool isMessagingEnabled,
      @_i5.required DateTime lastTranscriptReceivedAt,
      @_i5.required lastTranscript}) {
    assert(isHidden != null);
    assert(members != null);
    assert(isMessagingEnabled != null);
    assert(lastTranscriptReceivedAt != null);
    assert(lastTranscript != null);

    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "isHidden": _convertFirestoreStructure(isHidden, reference.path),
      "members": _convertFirestoreStructure(members, reference.path),
      "isMessagingEnabled":
          _convertFirestoreStructure(isMessagingEnabled, reference.path),
      "lastTranscriptReceivedAt":
          _convertFirestoreStructure(lastTranscriptReceivedAt, reference.path),
      "lastTranscript":
          _convertFirestoreStructure(lastTranscript, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => ShardRoomSnapshot(
            isHidden: isHidden,
            members: members,
            isMessagingEnabled: isMessagingEnabled,
            lastTranscriptReceivedAt: lastTranscriptReceivedAt,
            lastTranscript: lastTranscript));
  }

  Future<ShardRoomSnapshot> update(
      {bool isHidden,
      List<String> members,
      bool isMessagingEnabled,
      DateTime lastTranscriptReceivedAt,
      lastTranscript}) {
    return reference.updateData({
      "isHidden": isHidden,
      "members": members,
      "isMessagingEnabled": isMessagingEnabled,
      "lastTranscriptReceivedAt": lastTranscriptReceivedAt,
      "lastTranscript": lastTranscript
    }).then((_) => getSnapshot());
  }

  transcriptsCollection get transcripts {
    return transcriptsCollection(reference.collection("transcripts"));
  }
}

class ShardRoomSnapshot {
  const ShardRoomSnapshot(
      {@_i5.required this.isHidden,
      @_i5.required this.members,
      @_i5.required this.isMessagingEnabled,
      @_i5.required this.lastTranscriptReceivedAt,
      @_i5.required this.lastTranscript})
      : assert(isHidden != null),
        assert(members != null),
        assert(isMessagingEnabled != null),
        assert(lastTranscriptReceivedAt != null),
        assert(lastTranscript != null);

  factory ShardRoomSnapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return ShardRoomSnapshot(
          isHidden: _convertDartType(documentSnapshot["isHidden"]),
          members: _convertDartType(documentSnapshot["members"]),
          isMessagingEnabled:
              _convertDartType(documentSnapshot["isMessagingEnabled"]),
          lastTranscriptReceivedAt:
              _convertDartType(documentSnapshot["lastTranscriptReceivedAt"]),
          lastTranscript: _convertDartType(documentSnapshot["lastTranscript"]));
    } else {
      return null;
    }
  }

  final bool isHidden;

  final List<String> members;

  final bool isMessagingEnabled;

  final DateTime lastTranscriptReceivedAt;

  final lastTranscript;
}

class transcriptsCollection {
  transcriptsCollection(this.reference) : assert(reference != null);

  factory transcriptsCollection.fromPath(String path) {
    return transcriptsCollection(_firestore.collection(path));
  }

  final _i1.CollectionReference reference;

  String get id => reference.id;
  String get path => reference.path;
  TranscriptDocument documentById(String id) {
    return TranscriptDocument(reference.document(id));
  }
}

class TranscriptDocument {
  TranscriptDocument(this.reference) : assert(reference != null);

  factory TranscriptDocument.fromPath(String path) {
    return TranscriptDocument(_firestore.document(path));
  }

  final _i1.DocumentReference reference;

  String get documentId => reference.documentId;
  String get path => reference.path;
  transcriptsCollection get parent => transcriptsCollection(reference.parent());
  Future<TranscriptSnapshot> getSnapshot() {
    return reference.get().then((s) => TranscriptSnapshot.fromSnapshot(s));
  }

  Stream<TranscriptSnapshot> snapshotUpdates(
      {bool includeMetadataChanges = false}) {
    return reference
        .snapshots(includeMetadataChanges: includeMetadataChanges)
        .map((s) => TranscriptSnapshot.fromSnapshot(s));
  }

  Future<void> delete() {
    return reference.delete();
  }

  Future<TranscriptSnapshot> create(
      {@_i5.required String from, @_i5.required String text}) {
    assert(from != null);
    assert(text != null);

    final now = DateTime.now();
    final createdAt = now;
    final modifiedAt = now;
    final data = <String, Future<dynamic>>{
      "from": _convertFirestoreStructure(from, reference.path),
      "text": _convertFirestoreStructure(text, reference.path),
      "createdAt": _convertFirestoreStructure(createdAt, reference.path),
      "modifiedAt": _convertFirestoreStructure(modifiedAt, reference.path)
    };
    return Future.wait(data.values)
        .then((values) => Map.fromIterables(data.keys, values))
        .then((data) => reference.setData(data))
        .then((_) => TranscriptSnapshot(
            from: from,
            text: text,
            createdAt: createdAt,
            modifiedAt: modifiedAt));
  }

  Future<TranscriptSnapshot> update({String from, String text}) {
    return reference.updateData({
      "from": from,
      "text": text,
      "modifiedAt": DateTime.now()
    }).then((_) => getSnapshot());
  }
}

class TranscriptSnapshot {
  const TranscriptSnapshot(
      {@_i5.required this.from,
      @_i5.required this.text,
      @_i5.required this.createdAt,
      @_i5.required this.modifiedAt})
      : assert(from != null),
        assert(text != null),
        assert(createdAt != null),
        assert(modifiedAt != null);

  factory TranscriptSnapshot.fromSnapshot(
      _i1.DocumentSnapshot documentSnapshot) {
    if (documentSnapshot.exists) {
      return TranscriptSnapshot(
          from: _convertDartType(documentSnapshot["from"]),
          text: _convertDartType(documentSnapshot["text"]),
          createdAt: _convertDartType(documentSnapshot["createdAt"]),
          modifiedAt: _convertDartType(documentSnapshot["modifiedAt"]));
    } else {
      return null;
    }
  }

  final String from;

  final String text;

  final DateTime createdAt;

  final DateTime modifiedAt;
}
